#### CREATE ####
# Test cases related to creation of various efficiency targets
require 'spec_helper'

module RightsizingFrameworkTest
  describe 'RightsizingFramework createRightsizingEfficiencyTarget' do
    include_context 'request metadata'

    efficiency_targets = YAML.load_file(File.expand_path('../lib/support/yml/efficiency_targets.yml', __dir__))

    # Kubernetes Efficiency targets

    # 1 - Test case to create efficiency target for individual metrics of average type
    efficiency_targets[:Efficiency_targets][:KUBERNETES][:Average].each do |tc|
      it "KUBERNETES : Create efficiency target with Metric #{tc['metricName']}", monitoring: true do
        wrap_request do
          et_name = "#{tc['metricName']}#{Random.new.rand}"
          variables = {
            input: {
              assetType: 'KUBERNETES_CONTAINER',
              metrics: [tc],
              name: et_name
            }
          }
          graphql url(service: service),
                  format_request(
                    :createRightsizingEfficiencyTarget,
                    service: service
                  ),
                  variables, headers
          expect_success_graphql
          parsed_body = JSON.parse(response.body)
          id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
          expect(parsed_body["data"]["name"]).eql?(et_name)
          expect(parsed_body["data"]["assetType"]).eql?('KUBERNETES_CONTAINER')
          expect(parsed_body["data"]["id"]).eql?(!nil)
          expect(parsed_body["data"]["aggregation"]).eql?('avg')
          expect(parsed_body["data"]["metricName"]).eql?(tc['metricName'])
          expect(parsed_body["data"]["lowerRange"]).eql?(tc['lowerRange'])
          expect(parsed_body["data"]["upperRange"]).eql?(tc['upperRange'])
          # TODO: handle format checking at the end
          # check(:createRightsizingEfficiencyTarget, service: service)
          delete_ET(id)
        end
      end
    end

    # 2 - Test case to create efficiency target for individual metrics of maximum type
    efficiency_targets[:Efficiency_targets][:KUBERNETES][:Maximum].each do |tc|
      it "KUBERNETES : Create efficiency target with Metric #{tc['metricName']}", monitoring: true do
        wrap_request do
          et_name = "#{tc['metricName']}#{Random.new.rand}"
          variables = {
            input: {
              assetType: 'KUBERNETES_CONTAINER',
              metrics: [tc],
              name: et_name
            }
          }
          graphql url(service: service),
                  format_request(
                    :createRightsizingEfficiencyTarget,
                    service: service
                  ),
                  variables, headers
          expect_success_graphql
          id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
          expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                      name: et_name
          # TODO: handle format checking at the end
          # ToDo: Add the 'avg_network_out_pct' in efficiency target.yml and test the file - OP-135 must be fixed
          delete_ET(id)
        end
      end
    end

    # 3 - Test case to create efficiency target for metrics with average and max both
    it 'KUBERNETES : Create efficiency target with Metrics average and max both' do
      et_name = "Test_AVG_MAX_Both#{Random.new.rand}"
      wrap_request do
        variables = {
          input: {
            assetType: 'KUBERNETES_CONTAINER',
            metrics: efficiency_targets[:Efficiency_targets][:KUBERNETES][:Maximum_And_Average],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        # check(:createRightsizingEfficiencyTarget, service: service)
        delete_ET(id)
      end
    end

    # 4 - Test case to create efficiency target for all metrics together of average type
    it 'KUBERNETES : Create efficiency target with all the average metrics', monitoring: true do
      wrap_request do
        et_name = "Test_Efficiency_AVERAGE_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'KUBERNETES_CONTAINER',
            metrics: efficiency_targets[:Efficiency_targets][:KUBERNETES][:Average],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

    # 5 - Test case to create efficiency target with all metrics together of type MAX
    it 'KUBERNETES : Create efficiency target with all metrics maximum metrics ', monitoring: true do
      wrap_request do
        et_name = "Test_Custom_Efficiency_MAX_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'KUBERNETES_CONTAINER',
            metrics: efficiency_targets[:Efficiency_targets][:KUBERNETES][:Maximum],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

    # 6 - Test case to create efficiency target with metrics of same data type
    it 'KUBERNETES : Create efficiency target with both metrics of same Datatype ', monitoring: true do
      wrap_request do
        et_name = "Test_Custom_Efficiency_both_same_data#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'KUBERNETES_CONTAINER',
            metrics: efficiency_targets[:Efficiency_targets][:KUBERNETES][:Negative_1_Same_data],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_status(200)
        json = JSON.parse(response.body)
        message_ex = json['errors'][0]['message']
        expect(json['errors'][0]['message'].eql?("Average and Maximum aggregation type metrics together cannot be created and metric corresponding to a aggregation type has to be created only once"))
        expect(json['errors'][0]['extensions']['classification'].eql?("ValidationError"))
      end
    end

    # 7 - Test case to create efficiency target with metrics of no data
    it 'KUBERNETES : Create efficiency target with aggregation missing ', monitoring: true do
      wrap_request do
        et_name = "Test_Custom_Efficiency_Aggregation_missing#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'KUBERNETES_CONTAINER',
            metrics: efficiency_targets[:Efficiency_targets][:KUBERNETES][:Negative_2_No_data],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        raise('expected graphql error return, but no errors') unless
          json_body.key?(:errors)

        puts("Error is : #{json_body}")
      end
    end

    # 8 - Test case to create efficiency target with metrics of lowercase data
    it 'KUBERNETES : Create efficiency target with aggregation of lower case ', monitoring: true do
      et_name = "Test_Custom_Efficiency_with_aggregation_lowerCase#{Random.new.rand}"
      wrap_request do
        variables = {
          input: {
            assetType: 'KUBERNETES_CONTAINER',
            metrics: efficiency_targets[:Efficiency_targets][:KUBERNETES][:Negative_3_lowercase_data],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        raise('expected graphql error return, but no errors') unless
          json_body.key?(:errors)
      end
    end

    # 9 - Test case to create efficiency target with metrics of invalid data
    it 'KUBERNETES : Create efficiency target with metrics of invalid data', monitoring: true do
      wrap_request do
        et_name = "Test_Custom_Efficiency_with_invalid_metrics#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'KUBERNETES_CONTAINER',
            metrics: efficiency_targets[:Efficiency_targets][:KUBERNETES][:Negative_4_Invalid_data],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_status(200)
        json = JSON.parse(response.body)
        message_ex = json['errors'][0]['message']
        expect(json['errors'][0]['message'].eql?("Metric range has to be between 1 and 120"))
        expect(json['errors'][0]['extensions']['classification'].eql?("ValidationError"))
      end
    end

    # 11 - Test case to create efficiency target with metrics of multiple invalid inputs
    it 'KUBERNETES : Create efficiency target with invalid lower and upper ranges', monitoring: true do
      wrap_request do
        et_name = "Test_Custom_Efficiency_multiple_invalid_data#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'KUBERNETES_CONTAINER',
            metrics: efficiency_targets[:Efficiency_targets][:KUBERNETES][:Negative_5_Range],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_status(200)
        json = JSON.parse(response.body)
        message_ex = json['errors'][0]['message']
        expect(json['errors'][0]['message'].eql?("Metric range has to be between 1 and 120"))
        expect(json['errors'][0]['extensions']['classification'].eql?("ValidationError"))
      end
    end

    # 13 - Test case to validate creation of ET without metrics
    it 'KUBERNETES : Create efficiency target without metrics', monitoring: true do
      wrap_request do
        et_name = "Test_Custom_Efficiency_without_Metrics#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'KUBERNETES_CONTAINER',
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        raise('expected graphql error return, but no errors') unless
          json_body.key?(:errors)
      end
    end

    #14 Create ET of same name in RDS and EC2
    it 'KUBERNETES : Create efficiency target with the same name for two different assets', monitoring: true do
      wrap_request do
        et_name = "Test_Custom_Efficiency_without_Metrics#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'KUBERNETES_CONTAINER',
            name: et_name,
            metrics: [
              {
                metricName: 'avg_cpu',
                aggregation: 'AVG',
                lowerRange: '30',
                upperRange: '70'
              }
            ]
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        expect_status(200)
        parsed_body = JSON.parse(response.body)
        id_rds= parsed_body["data"]["id"]
        variables = {
          input: {
            assetType: 'AWS_EC2',
            name: et_name,
            metrics: [
              {
                metricName: 'avg_cpu',
                aggregation: 'AVG',
                lowerRange: '30',
                upperRange: '70'
              }
            ]
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        expect_status(200)
        parsed_body = JSON.parse(response.body)
        id_ec2= parsed_body["data"]["id"]
        fail('different id are not generated') if id_ec2!= id_rds
      end
    end

    #EBS Efficiency_targets
    it 'EBS : Create efficiency target with Metrics both max ', monitoring: true do
      et_name = "Test_AVG_MAX_Both#{Random.new.rand}"
      wrap_request do
        variables = {
          input: {
            assetType: 'AWS_EBS',
            metrics: efficiency_targets[:Efficiency_targets][:EBS][:both_max],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

    it 'EBS : Create efficiency target with Metrics average and max both', monitoring: true do
      et_name = "Test_AVG_MAX_Both#{Random.new.rand}"
      wrap_request do
        variables = {
          input: {
            assetType: 'AWS_EBS',
            metrics: efficiency_targets[:Efficiency_targets][:EBS][:Max_and_Avg],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

    it 'EBS : Create efficiency target with invalid aggregation', monitoring: true do
      wrap_request do
        et_name = "Test_Custom_Efficiency_multiple_invalid_data#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'AWS_EBS',
            metrics: efficiency_targets[:Efficiency_targets][:EBS][:Invalid_aggregation],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_status(200)
        json = JSON.parse(response.body)
        message_ex = json['errors'][0]['message']
        expect(json['errors'][0]['message'].eql?("Metric range has to be between 1 and 120")) #add the right error here
        expect(json['errors'][0]['extensions']['classification'].eql?("ValidationError"))
      end
    end


    # EC2 Efficiency_targets

    # 3 - Test case to create efficiency target for metrics with average and max both
    it 'EC2 : Create efficiency target with Metrics average and max both', monitoring: true do
      et_name = "Test_AVG_MAX_Both#{Random.new.rand}"
      wrap_request do
        variables = {
          input: {
            assetType: 'AWS_EC2',
            metrics: efficiency_targets[:Efficiency_targets][:EC2][:Maximum_And_Average],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        # check(:createRightsizingEfficiencyTarget, service: service)
        delete_ET(id)
      end
    end

    # Duplicate of 1t Test case
    # 4 - Test case to create efficiency target for all metrics together of average type
    it 'EC2 : Create efficiency target with all the average metrics', monitoring: true do
      wrap_request do
        et_name = "Test_Efficiency_AVERAGE_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'AWS_EC2',
            metrics: efficiency_targets[:Efficiency_targets][:EC2][:Average],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

    # 5 - Test case to create efficiency target with all metrics together of type MAX
    it 'EC2 : Create efficiency target with all metrics maximum metrics ', monitoring: true do
      wrap_request do
        et_name = "Test_Custom_Efficiency_MAX_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'AWS_EC2',
            metrics: efficiency_targets[:Efficiency_targets][:EC2][:Maximum],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

    # RDS Efficiency_targets

    it 'RDS : Create efficiency target with Metrics average and max both', monitoring: true do
      et_name = "Test_AVG_MAX_Both#{Random.new.rand}"
      wrap_request do
        variables = {
          input: {
            assetType: 'AWS_RDS',
            metrics: efficiency_targets[:Efficiency_targets][:RDS][:Maximum_And_Average],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        # check(:createRightsizingEfficiencyTarget, service: service)
        delete_ET(id)
      end
    end

    it 'RDS : Create efficiency target with all the average metrics' do
      wrap_request do
        et_name = "Test_Efficiency_AVERAGE_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'AWS_RDS',
            metrics: efficiency_targets[:Efficiency_targets][:RDS][:Average],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

    it 'RDS : Create efficiency target with all metrics maximum metrics ', monitoring: true do
      wrap_request do
        et_name = "Test_Custom_Efficiency_MAX_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'AWS_RDS',
            metrics: efficiency_targets[:Efficiency_targets][:RDS][:Maximum],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

    # Elasticache Efficiency_targets

    it 'ELASTICACHE : Create efficiency target with Metrics average and max both', monitoring: true do
      et_name = "Test_AVG_MAX_Both#{Random.new.rand}"
      wrap_request do
        variables = {
          input: {
            assetType: 'AWS_RDS',
            metrics: efficiency_targets[:Efficiency_targets][:ELASTICACHE][:Maximum_And_Average],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        # check(:createRightsizingEfficiencyTarget, service: service)
        delete_ET(id)
      end
    end

    it 'ELASTICACHE : Create efficiency target with all the average metrics', monitoring: true do
      wrap_request do
        et_name = "Test_Efficiency_AVERAGE_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'AWS_ELASTICACHE',
            metrics: efficiency_targets[:Efficiency_targets][:ELASTICACHE][:Average],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

    it 'ELASTICACHE : Create efficiency target with all metrics maximum metrics ', monitoring: true do
      wrap_request do
        et_name = "Test_Custom_Efficiency_MAX_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'AWS_ELASTICACHE',
            metrics: efficiency_targets[:Efficiency_targets][:ELASTICACHE][:Maximum],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

# DynamoDB Efficiency Target tests
    it 'DynamoDB : Create efficiency target with Metrics average and max both', monitoring: true do
      et_name = "Test_AVG_MAX_Both#{Random.new.rand}"
      wrap_request do
        variables = {
          input: {
            assetType: 'AWS_DYNAMO_DB',
            metrics: efficiency_targets[:Efficiency_targets][:DYNANMODB][:Maximum_And_Average],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        # check(:createRightsizingEfficiencyTarget, service: service)
        delete_ET(id)
      end
    end

    it 'DynamoDB : Create efficiency target with all the average metrics', monitoring: true do
      wrap_request do
        et_name = "Test_Efficiency_AVERAGE_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'AWS_DYNAMO_DB',
            metrics: efficiency_targets[:Efficiency_targets][:DYNANMODB][:Average],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

    it 'DynamoDB : Create efficiency target with all metrics maximum metrics ', monitoring: true do
      wrap_request do
        et_name = "Test_Custom_Efficiency_MAX_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'AWS_DYNAMO_DB',
            metrics: efficiency_targets[:Efficiency_targets][:DYNANMODB][:Maximum],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

# Azure VM Efficiency_targets

    it 'AZURE : Create efficiency target with Metrics average and max both', monitoring: true do
      et_name = "Test_AVG_MAX_Both#{Random.new.rand}"
      wrap_request do
        variables = {
          input: {
            assetType: 'AZURE_VM',
            metrics: efficiency_targets[:Efficiency_targets][:AZURE][:Maximum_And_Average],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        # check(:createRightsizingEfficiencyTarget, service: service)
        delete_ET(id)
      end
    end

    # 4 - Test case to create efficiency target for all metrics together of average type
    it 'Azure : Create efficiency target with all the average metrics', monitoring: true do
      wrap_request do
        et_name = "Test_Efficiency_AVERAGE_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'AZURE_VM',
            metrics: efficiency_targets[:Efficiency_targets][:AZURE][:Average],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

    # 5 - Test case to create efficiency target with all metrics together of type MAX
    it 'Azure : Create efficiency target with all metrics maximum metrics ' do
      wrap_request do
        et_name = "Test_Custom_Efficiency_MAX_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'AZURE_VM',
            metrics: efficiency_targets[:Efficiency_targets][:AZURE][:Maximum],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

    # 6 - Test case to create efficiency target SQL VCore Max
    it 'Azure SQL Vcore: Create efficiency target with all metrics maximum metrics ' do
      wrap_request do
        et_name = "Test_Custom_Efficiency_MAX_ALL#{Random.new.rand}"
        variables = {
          input: {
            assetType: 'AZURE_SQL_VCORE',
            metrics: efficiency_targets[:Efficiency_targets][:AZURE_SQL_VCORE][:Maximum],
            name: et_name
          }
        }
        graphql url(service: service),
                format_request(
                  :createRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        id = json_body[:data][:createRightsizingEfficiencyTarget][:efficiencyTarget][:id]
        expect_json 'data.createRightsizingEfficiencyTarget.efficiencyTarget',
                    name: et_name
        delete_ET(id)
      end
    end

  end
end

