require 'spec_helper'
require 'date'

module RightsizingFrameworkTest
  describe 'RightsizingFramework GetRecommendations' do
    include_context 'request metadata'

    # Test -1 : Validate if a user is able to get the terminate recommendation
    xit 'Validate the recommendation for system defined efficiency target and 7 days evaluation duration for termination of an instance' do
      args_get_recommendations = {
        assetType: 'AZURE_VM',
        efficiencyTarget: 'crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: true
      }

      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      azure_instance_id = '/subscriptions/700f3a5c-8c56-44b9-aa1d-fcfb1493aa4b/resourcegroups/lucas/providers/microsoft.compute/virtualmachines/test'
      terminate_recommendation = response_recommendations[:edges].select { |x| x[:node][:targetAsset][:azureInstanceId].eql?(azure_instance_id)}[0][:node][:terminateRecommendation]
      expect(terminate_recommendation).to be true #verifying the terminate is true
      end

    # Test -2 : Validate burstable instances -false
    it 'Validate burstable instances -false', monitoring: true do
      args_get_recommendations = {
        assetType: 'AZURE_VM',
        efficiencyTarget: 'crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_30_DAYS',
        returnAllInstances: false,
        processingFlagsMap: [
          {
            "key": "recommendBurstable",
            "value": "false"
          }
        ],
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      burstable_array = %w[Standard_B1s Standard_B1ms Standard_B2s Standard_B2ms Standard_B4ms Standard_B8ms Standard_B12ms Standard_B16ms Standard_B20ms]
      recommended_instances = response_recommendations[:edges].map { |r| r[:node][:options] }.flatten.map { |o| o[:instanceType][:name] }
      expect(burstable_array & recommended_instances).to be_empty
        end




    # Test -3 : Validate burstable instances -true
    xit 'Validate burstable instances - true' do
      args_get_recommendations = {
        assetType: 'AZURE_VM',
        efficiencyTarget: 'crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_30_DAYS',
        returnAllInstances: false,
        processingFlagsMap: [
          {
            "key": "recommendBurstable",
            "value": "true"
          }
        ],
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      burstable_array = %w[Standard_B1s Standard_B1ms Standard_B2s Standard_B2ms Standard_B4ms Standard_B8ms Standard_B12ms Standard_B16ms Standard_B20ms]
      recommended_instances = response_recommendations[:edges].map { |r| r[:node][:options] }.flatten.map { |o| o[:instanceType][:name] }
      expect(burstable_array & recommended_instances).not_to be_empty
    end

    #test -4 validate using availability filter
    # will enable it, need some more info on it
    xit 'Validate filter by availability zone', monitoring: true do
      args_get_recommendations_without_filter = {
        assetType: 'AZURE_VM',
        efficiencyTarget: 'crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_30_DAYS',
        returnAllInstances: true
      }
      response_recommendations_without_filter = rightsizing_recommendations(args_get_recommendations_without_filter)
      total_edges=response_recommendations_without_filter[:edges].length
      args_get_recommendations = {
        assetType: 'AZURE_VM',
        efficiencyTarget: 'crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_30_DAYS',
        filterOptions:
          {
            otherCriteria: [
              {
                name: 'azure_zone_id',
                values: ['East US 2']
              }
            ]
          },
        returnAllInstances: true
      }
      response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
      expect(response_recommendations_call[:edges].length.eql?total_edges).to be false
        end

    # Validate using quick filter

    # test -5 Validate show all
    it 'Validate show ALL is false', monitoring: true do
      args_get_recommendations = {
        assetType: 'AZURE_VM',
        efficiencyTarget: 'crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_30_DAYS',
        returnAllInstances: false
      }
      response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
      expect(response_recommendations_call[:edges].all? { |r| r[:node][:options].length > 0 }).to be true
    end

    #test -6 validate show ALL true
    it 'Validate show ALL is true' do
      args_get_recommendations = {
        assetType: 'AZURE_VM',
        efficiencyTarget: 'crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_30_DAYS',
        returnAllInstances: true,
        after: nil
      }
      response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
      count = response_recommendations_call[:edges].select { |x| x[:node][:options].length == 0 }.length
      while(response_recommendations_call[:pageInfo][:hasNextPage]) do
        args_get_recommendations[:after] = response_recommendations_call[:pageInfo][:endCursor]
        response_recommendations_call = rightsizing_recommendations(args_get_recommendations)
        count += response_recommendations_call[:edges].select { |x| x[:node][:options].length == 0 }.length
      end
      expect(count).not_to eql(0)
    end

    # test -7 Validate comibination of availability and quick filter
    it 'Validate the quick filter of multiple utilization status ' do
      other_criteria = [
        {
          name: 'currentUtilizationStatus',
          values: %w[GOOD_FIT UNDER_TARGET]
        }
      ]
      args_get_recommendations = {
        assetType: 'AZURE_VM',
        efficiencyTarget: 'crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_max',
        evaluationDuration: 'LAST_30_DAYS',
        filterOptions: {
          otherCriteria: other_criteria
        },
        returnAllInstances: true
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      expect(response_recommendations_call_1[:edges].all? { |r| other_criteria[0][:values].include? r[:node][:currentUtilizationStatus] }).to be true
    end

    # test -8 Validate current metrics, options length, and count of classifications equal to total
    xit 'Validate current metrics, options length, and count of classifications equal to total' do
      args_get_recommendations = {
        assetType: 'AZURE_VM',
        efficiencyTarget: 'crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_7_DAYS',
        returnAllInstances: false
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      expect((response_recommendations[:edges].size())>0).to be true
      expect([true, false].include?response_recommendations[:edges][0][:node][:terminateRecommendation]).to be true
      expect(response_recommendations[:edges][0][:node][:options]).to_not be_nil
      expect(response_recommendations[:edges][0][:node][:currentMetrics].size().eql?(3)). to be true
      expect(response_recommendations[:edges][0][:node][:currentMetrics][0][:metricName]).to eq("avg_cpu")
      expect(response_recommendations[:edges][0][:node][:currentMetrics][0][:aggregation]).to eq("AVG")
      expect(response_recommendations[:edges][0][:node][:currentMetrics][1][:metricName]).to eq("avg_mem")
      expect(response_recommendations[:edges][0][:node][:currentMetrics][2][:metricName]).to eq("avg_disk")
      expect(response_recommendations[:edges][0][:node][:currentMetrics][1][:aggregation]).to eq("AVG")
      expect(response_recommendations[:edges][0][:node][:currentMetrics][2][:aggregation]).to eq("AVG")
      expect(response_recommendations[:edges].all? { |r| r[:node][:options].length > 3 && r[:node][:options].length <=0 }).to be false
      expect(response_recommendations[:utilizationStatusSummary].values.reject{ |i| i.kind_of?(String)}.sum()).to eq(response_recommendations[:totalCount])
    end


    # Test - 9 : Validate best recommendation price is the lowest among the recommendation options
    it 'Validate the first best recommendation is of the lowest cost among the recommendations' do
      args_get_recommendations = {
        assetType: 'AZURE_VM',
        efficiencyTarget: 'crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_30_DAYS',
        returnAllInstances: false
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      response_recommendations[:edges].each do |edge|
        array_of_projectedmonthly=edge[:node][:options].map {|o| o[:projectedMonthlyPrice]}
        projectedMonthlyPrice = edge[:node][:options].select { |o| o[:projectedMonthlyPrice] if o[:bestFit].eql?(true) }[0][:projectedMonthlyPrice]
        expect(projectedMonthlyPrice).to eq(array_of_projectedmonthly.min)
      end
      end

    #  Test - 10 : Validate that the savings is the difference of the current and the projected price
    xit 'Validate saving of an instance is the difference of current price and the estimated price' do
      args_get_recommendations = {
        assetType: 'AZURE_VM',
        efficiencyTarget: 'crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg',
        evaluationDuration: 'LAST_30_DAYS',
        returnAllInstances: true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      response_recommendations[:edges].each do |edge|
        expect((edge[:node][:currentMonthlyPrice]- edge[:node][:projectedMonthlyPrice]).round(2).eql?(edge[:node][:projectedMonthlySavings].round(2))).to be true
      end
    end
  end
  end




