# Test cases related to getting the recommendations

require 'spec_helper'
require 'date'

module RightsizingFrameworkTestDynamoDB
  describe 'RightsizingFramework GetRecommendations for DynamoDB' do
    include_context 'request metadata'

    # Test -1 : Validate the basic recommendatiossn API Response type
    it 'Validate the recommendation for system defined efficiency target and 7 days evaluation duration' do
      wrap_request do
            variables = {
               "requestInput": {
                 "assetType": "AWS_DYNAMO_DB",
                 "efficiencyTarget": "crn:0:rightsizing/AWS_DYNAMO_DB_EfficiencyTarget:system_avg",
                 "evaluationDuration": "LAST_7_DAYS",
                 "filterOptions": {},
                 "returnAllInstances": false
               }
            }
            graphql url(service: service),
                format_request(
                   :rightsizingRecommendations,
                   service: service
                ),
            variables,
            apitest_headers(with_auth: true, auth_service: auth_service),
            service: service
            expect_success_graphql
            expect_json_types 'data.rightsizingRecommendations',
                               edges: :array,
                               totalCount: :int
      end
    end

    # Test - 2 : Validate if a user is able to getting proper classification for Under target (Re-enable from xit to it after https://cloudhealthtech.atlassian.net/browse/OP-1624 this fix)
    xit 'Validate the rightsizing recommendation to check Under target instance' do
      other_criteria = [ {
                            "name": "currentUtilizationStatus",
                            "values": [
                              "UNDER_TARGET"
                            ]
                          }
                        ]
      args_get_recommendations = {
              assetType: 'AWS_DYNAMO_DB',
              efficiencyTarget: 'crn:0:rightsizing/AWS_DYNAMO_DB_EfficiencyTarget:system_avg',
              evaluationDuration: 'LAST_30_DAYS',
              filterOptions: {
                otherCriteria: other_criteria
              },
              returnAllInstances: false
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      expect(response_recommendations[:edges].all? { |r| other_criteria[0][:values].include? r[:node][:currentUtilizationStatus] }).to be true
    end

    # Test - 3 : Validate Filter with respect to Cluster  ( Re-enable from xit to it after https://cloudhealthtech.atlassian.net/browse/OP-1624 this fix)
    xit 'Validate the rightsizing recommendation to check Filter with Account' do
      other_criteria = [ {
                            "name": "aws_account_id",
                            "values": [
                              "844584796563"
                            ]
                          }
                        ]
      args_get_recommendations = {
              assetType: 'AWS_DYNAMO_DB',
              efficiencyTarget: 'crn:0:rightsizing/AWS_DYNAMO_DB_EfficiencyTarget:system_avg',
              evaluationDuration: 'LAST_30_DAYS',
              filterOptions: {
                otherCriteria: other_criteria
              },
              returnAllInstances: false
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      expect(response_recommendations[:edges].all? { |r| other_criteria[0][:values].include? r[:node][:targetAsset][:account][:accountId] }).to be true
    end
  end
end
