require 'spec_helper'

module RightsizingFrameworkTest
  describe 'RightsizingFramework GetRecommendationsSummary' do
    include_context 'request metadata'

    # Test 1 Validating summary response for assets which has same response types

    assetType = Array.[]('AWS_EC2', 'AWS_RDS','AZURE_VM', 'AWS_OPEN_SEARCH','AWS_EBS','AWS_ELASTICACHE','AWS_DYNAMO_DB', 'AZURE_SQL_VCORE', 'AZURE_SQL_DTU')
    assetType.each do |asset|
      it 'Validate Basic response for ' + asset + ' Summary API' do
        wrap_request do
          variables = {
            'request': {
               'assetType': asset,
               'efficiencyTarget': 'crn:0:rightsizing/' + asset + '_EfficiencyTarget:system_avg',
               'evaluationDuration': 'LAST_7_DAYS'
            }
          }
          graphql url(service: service),
              format_request(
                 :rightsizingRecommendationsSummary,
                 service: service
              ),
              variables,
              apitest_headers(with_auth: true, auth_service: auth_service),
              service: service
              expect_success_graphql
              expect_json_types 'data.rightsizingRecommendationsSummary',
                                 monthlyCost: :float,
                                 projectedMonthlyPrice: :float,
                                 projectedMonthlyDifference: :float,
                                 totalRecommendations: :int,
                                 resizeRecommendationsCount: :int,
                                 terminateRecommendationsCount: :int
        end
      end
    end

      # Test 2 Validating summary response for K8s which is different response
      it 'Validate Basic response for K8s Summary API' do
        wrap_request do
          variables = {
            'request': {
               'assetType': 'KUBERNETES_CONTAINER',
               'efficiencyTarget': 'crn:0:rightsizing/KUBERNETES_CONTAINER_EfficiencyTarget:system_avg',
               'evaluationDuration': 'LAST_7_DAYS'
            }
          }
          graphql url(service: service),
              format_request(
                 :rightsizingRecommendationsSummary,
                 service: service
              ),
              variables,
              apitest_headers(with_auth: true, auth_service: auth_service),
              service: service
              expect_success_graphql
              expect_json_types 'data.rightsizingRecommendationsSummary',
                                 requests: :array,
                                 recommended: :array,
                                 totalRecommendations: :int,
                                 resizeRecommendationsCount: :int,
                                 noChangeRecommendationsCount: :int
        end
      end


    # Test 3 Validate Recommendation API Count matches with Summary API for same response type assets

    assetType = Array.[]( 'AWS_EC2', 'AWS_RDS','AZURE_VM','AWS_OPEN_SEARCH','AWS_EBS','AWS_ELASTICACHE','AWS_DYNAMO_DB','AZURE_SQL_VCORE','AZURE_SQL_DTU')
    assetType.each do |asset|
      it 'Validate the values of totalRecommendation, resizeRecommendationsCount, terminateRecommendationsCount with ' + asset + ' Summary' do
        et_id = 'crn:0:rightsizing/' + asset +'_EfficiencyTarget:system_avg'
        args_get_recommendations = {
          assetType: asset,
          efficiencyTarget: et_id,
          evaluationDuration: 'LAST_60_DAYS',
          returnAllInstances: true,
          after: nil,
          sortRules: [
            {
              "field": "projectedMonthlyPrice",
              "direction": "DESC"
            }
          ]
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        termination_count = response_recommendations_call_1[:edges].select { |x| x[:node][:terminateRecommendation] == true && x[:node][:options].length == 0 }.length
        resize_count = response_recommendations_call_1[:edges].select { |x| x[:node][:options].length > 0 }.length

        while response_recommendations_call_1[:pageInfo][:hasNextPage].eql?(true)
          args_get_recommendations[:after] = response_recommendations_call_1[:pageInfo][:endCursor]
          response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
          termination_count += response_recommendations_call_1[:edges].select { |x| x[:node][:terminateRecommendation] == true && x[:node][:options].length == 0 }.length
          resize_count += response_recommendations_call_1[:edges].select { |x| x[:node][:options].length > 0 }.length
        end

        args_get_summary = {
          assetType: asset,
          efficiencyTarget: et_id,
          evaluationDuration: 'LAST_60_DAYS',
          returnAllInstances: true
        }
        response_summary = rightsizing_recommendations_summary(args_get_summary)

        expect(response_summary[:resizeRecommendationsCount]).to eql(resize_count)
        expect(response_summary[:terminateRecommendationsCount]).to eql(termination_count)
        expect(response_summary[:totalRecommendations]).to eql(termination_count + resize_count)
      end
    end

    # Test 4 Validate Recommendation API Count matches with Summary API for K8s which has different response with other assets
    # Test
    it 'Validate the values of total recommendation, resizeRecommendationsCount, noChangeRecommendationsCount with K8s Summary' do
            asset = 'KUBERNETES_CONTAINER'
            et_id = 'crn:0:rightsizing/' + asset +'_EfficiencyTarget:system_avg'
            args_get_recommendations = {
              assetType: asset,
              efficiencyTarget: et_id,
              evaluationDuration: 'LAST_30_DAYS',
              returnAllInstances: true,
              after: nil,
              filterOptions: {  # Currently for K8s This filter needs to be applied as we are supporting K8s RZ only for AWS
                        "otherCriteria": [
                            {
                                "name": "cloud_provider",
                                "values": [
                                    "AWS"
                                ]
                            }
                        ]
                    },
            }
            response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
            resize_count = response_recommendations_call_1[:edges].select { |x| x[:node][:options].length > 0 }.length
            nochange_count = response_recommendations_call_1[:edges].select { |x| x[:node][:options].length == 0 }.length

            while response_recommendations_call_1[:pageInfo][:hasNextPage].eql?(true)
              args_get_recommendations[:after] = response_recommendations_call_1[:pageInfo][:endCursor]
              response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
              nochange_count += response_recommendations_call_1[:edges].select { |x| x[:node][:options].length == 0 }.length
              resize_count += response_recommendations_call_1[:edges].select { |x| x[:node][:options].length > 0 }.length
            end

            args_get_summary = {
                assetType: asset,
                efficiencyTarget: et_id,
                evaluationDuration: 'LAST_30_DAYS',
                returnAllInstances: true,
                filterOptions: {
                                 "otherCriteria": [
                                   {
                                     "name": "cloud_provider",
                                     "values": [
                                       "AWS"
                                     ]
                                   }
                                 ]
                               }
            }
            response_summary = rightsizing_recommendations_summary(args_get_summary)

            expect(response_summary[:resizeRecommendationsCount]).to eql(resize_count)
            expect(response_summary[:noChangeRecommendationsCount]).to eql(nochange_count)
            expect(response_summary[:totalRecommendations]).to eql(nochange_count + resize_count)
        end

  end
end
