# Test cases related to getting the recommendations

require 'spec_helper'
require 'date'

module RightsizingFrameworkTestK8s
  describe 'RightsizingFramework GetRecommendations for K8s' do
    include_context 'request metadata'

    # Test -1 : Validate the basic recommendation API Response type
    xit 'Validate the recommendation for system defined efficiency target and 7 days evaluation duration' do
      wrap_request do
            variables = {
               "requestInput": {
                 "assetType": "KUBERNETES_CONTAINER",
                 "efficiencyTarget": "crn:0:rightsizing/KUBERNETES_CONTAINER_EfficiencyTarget:system_avg",
                 "evaluationDuration": "LAST_7_DAYS",
                 "filterOptions": {
                   "otherCriteria": [
                     {
                       "name": "cloud_provider",
                       "values": [
                         "AWS"
                       ]
                     }
                   ]
                 },
                 "returnAllInstances": false
               }
            }
            graphql url(service: service),
                format_request(
                   :rightsizingRecommendations,
                   service: service
                ),
            variables,
            apitest_headers(with_auth: true, auth_service: auth_service),
            service: service
            expect_success_graphql
            expect_json_types 'data.rightsizingRecommendations',
                               edges: :array,
                               totalCount: :int
      end
    end

    # Test - 2 : Validate if a user is able to getting proper classification for Under target
    it 'Validate the rightsizing recommendation to check Under target instance' do
      other_criteria = [ {
                            "name": "currentUtilizationStatus",
                            "values": [
                              "UNDER_TARGET"
                            ]
                          },
                          {
                            "name": "cloud_provider",
                            "values": [
                              "AWS"
                            ]
                          }
                        ]
      args_get_recommendations = {
              assetType: 'KUBERNETES_CONTAINER',
              efficiencyTarget: 'crn:0:rightsizing/KUBERNETES_CONTAINER_EfficiencyTarget:system_avg',
              evaluationDuration: 'LAST_30_DAYS',
              filterOptions: {
                otherCriteria: other_criteria
              },
              returnAllInstances: false
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      expect(response_recommendations[:edges].all? { |r| other_criteria[0][:values].include? r[:node][:currentUtilizationStatus] }).to be true
    end

    # Test - 3 : Validate Filter with respect to Cluster
    it 'Validate the rightsizing recommendation to check Filter with cluster' do
      other_criteria = [ {
                            "name": "cluster_id",
                            "values": [
                              "5772436045856"
                            ]
                          },
                          {
                            "name": "cloud_provider",
                            "values": [
                              "AWS"
                            ]
                          }
                        ]
      args_get_recommendations = {
              assetType: 'KUBERNETES_CONTAINER',
              efficiencyTarget: 'crn:0:rightsizing/KUBERNETES_CONTAINER_EfficiencyTarget:system_avg',
              evaluationDuration: 'LAST_30_DAYS',
              filterOptions: {
                otherCriteria: other_criteria
              },
              returnAllInstances: false
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      expect(response_recommendations[:edges].all? { |r| other_criteria[0][:values].include? r[:node][:targetAsset][:cluster][:clusterId] }).to be true
    end
  end
end
