# Test cases related to getting the recommendations

require 'spec_helper'
require 'date'

module RightsizingFrameworkTest
  describe 'RightsizingFramework GetRecommendations' do
    include_context 'request metadata'

    # Test -1 : Validate if a user is able to get the terminate recommendation
    # Chosen instance - "production-worker-cubes-jruby8xl-temp-92-1" instance id "i-0b6a60449ecf1470f"
    # ET - Average , Duration - Last 7
    xit 'Validate the recommendation for system defined efficiency target and 7 days evaluation duration for termination of an instance' do
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        'evaluationDuration': 'LAST_7_DAYS',
        'returnAllInstances': true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      instance_id = "i-05bf5b602f27e881e"
      cpu_value = get_metrics_value_based_on_userInput(instance_id, response_recommendations, "avg_cpu")
      mem_value = get_metrics_value_based_on_userInput(instance_id, response_recommendations, "avg_mem")
      expected_termination_recommendation = cpu_value < 1 && mem_value < 1 ? true : false
      termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, response_recommendations, nil)
      fail ("Recommendations are not matching the expectation ") unless termination_recommendation.eql?(expected_termination_recommendation)
    end

    # Test - 2 : Validate if a user is able to get the recommendations for termination
    # Chosen instance- instance id "i-0b5c8e0103bba1b18"
    # ET - Average , Duration - Last 60 days

    # Test - 3 : Validate the data for an instance is matching as expected from the API response
    xit 'Validate the data match of instance type and the instance id for an instance' do
      baseline_instance_id = 'i-00e6e8fdafa3dc697'
      baseline_instance_name = 'DO-NOT-DELETE-OPTIMUS-12-2023'
      baseline_type_of_instance = 'p2.xlarge'
      # sortRules = {'field': ":projectedMonthlySavings", 'direction': 'DESC'}
      metrics = [
        {
          'metricName': 'avg_cpu',
          'aggregation': 'AVG',
          'lowerRange': 2,
          'upperRange': 5
        },
        {
          'metricName': 'avg_mem',
          'aggregation': 'AVG',
          'lowerRange': 2,
          'upperRange': 5
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET7"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_60_DAYS',
        'returnAllInstances': true,
        #'sortRules': sortRules
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      map = get_map_from_responses(response_recommendations)
      instance_name_from_api = get_instance_name(map[baseline_instance_id])
      instance_type_from_api = get_instance_type(map[baseline_instance_id])
      fail("current instance name and current instance types are not matching") unless instance_name_from_api.eql?(baseline_instance_name) && instance_type_from_api.eql?(baseline_type_of_instance)
      #Clean up of the ET
      delete_ET(et_id)
    end

    # Test - 4 : Validate the utilization status of any instance
    it 'Validate utilization status for any instance' do
      lower_range = 2
      upper_range = 5
      metrics = [
        {
          'metricName': 'avg_cpu',
          'aggregation': 'AVG',
          'lowerRange': lower_range,
          'upperRange': upper_range
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET8"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_60_DAYS',
        'returnAllInstances': true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      utilization_status_from_api = response_recommendations[:edges][0][:node][:currentUtilizationStatus]
      instance_id = get_instance_id(response_recommendations[:edges][0])
      cpu_value = get_metrics_value_based_on_userInput(instance_id, response_recommendations, 'avg_cpu')
      expected_utilization_status = get_expected_utilization_status(cpu_value, lower_range, upper_range)
      fail("Utilization status is not matching") unless utilization_status_from_api.eql?(expected_utilization_status)
      #Clean up of the ET
      delete_ET(et_id)
    end

    # Test - 5 : Validate pagination is working as expected , here assumtion for the redshirt account is considered for instances being more than 100
    xit 'Validate pagination is working as expected', monitoring: true do
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        'evaluationDuration': 'LAST_7_DAYS',
        'returnAllInstances': true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      args_get_summary = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        'evaluationDuration': 'LAST_7_DAYS',
        'returnAllInstances': true
      }
      response_summary = rightsizing_recommendations_summary(args_get_summary)
      total_instances = response_summary[:totalTargetResources]
      if total_instances <= 100
        fail("Pagination is not working") unless total_instances.eql?(response_recommendations[:edges].size)
      else
        fail ("Pagination is not working") if total_instances.eql?(response_recommendations[:edges].size)
      end
    end

    #  Test - 6 : Validate that the savings is the difference of the current and the projected price
    xit 'Validate saving of an instance is the difference of current price and the estimated price' do
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        'evaluationDuration': 'LAST_30_DAYS',
        'returnAllInstances': true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      projectedMonthlyCost = response_recommendations[:edges][0][:node][:projectedMonthlyPrice]
      currentMonthlyCost = response_recommendations[:edges][0][:node][:currentMonthlyPrice]
      fail('Data is not matching') unless (currentMonthlyCost - projectedMonthlyCost).eql?(response_recommendations[:edges][0][:node][:projectedMonthlySavings])
    end

    # Test - 7 : Validate the total number of recommendations for an instance
    it 'Validate the number of recommendations for an instance', monitoring: true do
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        'evaluationDuration': 'LAST_30_DAYS',
        'returnAllInstances': true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      response_recommendations[:edges].each do |edge|
        if edge[:node][:options].length() > 3
          fail('More than three recommendations are available')
        end
      end
    end

    # Test - 8 : Validate best recommendation price is the lowest among the recommendation options
    it 'Validate the first best recommendation is of the lowest cost among the recommendations' do
      lower_range = 2
      upper_range = 20
      metrics = [
        {
          'metricName': 'avg_cpu',
          'aggregation': 'AVG',
          'lowerRange': lower_range,
          'upperRange': upper_range
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET9"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_30_DAYS',
        'returnAllInstances': true
      }
      required_node_options = []
      response_recommendations = rightsizing_recommendations(args_get_recommendations)

      response_recommendations[:edges].each do |edge|
        if (edge[:node][:options].length() > 1 && edge[:node][:options].length() <= 3)
          required_node_options = edge[:node][:options]
          break
        end
      end
      recommendations_price_array = []
      first_best_recommendation_cost = ''
      unless (required_node_options.size < 2 )
        required_node_options.each do |option|
          if(option[:projectedMonthlyPrice] && option[:bestFit].eql?(true))
            first_best_recommendation_cost = option[:instanceType][:projectedMonthlyPrice]
            recommendations_price_array.append(first_best_recommendation_cost)
          elsif(option[:projectedMonthlyPrice])
            recommendations_price_array.append(option[:instanceType][:projectedMonthlyPrice])
          end
        end
        recommendations_price_array = recommendations_price_array.sort
        fail("Best fit recommendation is not the lowest cost recommendation") unless first_best_recommendation_cost.eql?(recommendations_price_array[0])
      end
      #Clean up of the ET
      delete_ET(et_id)
    end

    # Test-9 - Validate the total number of instances of the classification available is equal to the number of instances for that particular account

    # Test-10 - Validate Projected savings must be based on the first best recommendation available.
    xit 'Validate Projected savings must be based on the first best recommendation available', monitoring: true do
      lower_range = 2
      upper_range = 20
      metrics = [
        {
          'metricName': 'avg_mem',
          'aggregation': 'AVG',
          'lowerRange': lower_range,
          'upperRange': upper_range
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET10"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_60_DAYS',
        'returnAllInstances': true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      first_best_recommendation_price = 0
      price_from_api = 0
      response_recommendations[:edges].each do |edge|
        if (edge[:node][:options].length() >= 1)
          edge[:node][:options].each do |option|
            if option[:bestFit].eql?(true)
              first_best_recommendation_price = option[:instanceType][:onDemandPrice]
              price_from_api = option[:instanceType][:onDemandPriceMonthlyCost]
              break
            end
          end
          break
        end
      end
      time = Time.new
      calculated_price_from_first_best = first_best_recommendation_price * 24 * days_in_month(time.month)
      fail("price not matching") unless calculated_price_from_first_best.round(2).eql?(price_from_api.round(2))
      #Clean up of the ET
      delete_ET(et_id)
    end

    # Test- 11 Validate the scenario with two instances of same metrics but just different instance IDs provide same recommendations.
    # Expecting m6g.large as the first best recommendation for both
    # Bug
    xit 'Validate the scenarios with two instances of same metrics but just different instance IDs provide same recommendations', monitoring: true do
      instance_id_1 = "i-0351d7be68169b673"
      instance_id_2 = "i-0d20a70a6cf10a7d3"
      lower_range = 2
      upper_range = 20
      metrics = [
        {
          'metricName': 'avg_mem',
          'aggregation': 'AVG',
          'lowerRange': lower_range,
          'upperRange': upper_range
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET11"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_60_DAYS',
        'returnAllInstances': true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      best_recommendation_1 = get_first_best_recommendation(instance_id_1, response_recommendations, nil)
      best_recommendation_2 = get_first_best_recommendation(instance_id_2, response_recommendations, nil)
      fail("Even though metrics of the the two instances fall into same range and are similar, their recommendations are not same") unless best_recommendation_1.eql?(best_recommendation_2)
      #Clean up of the ET
      delete_ET(et_id)
    end

    #Test-12 Validate whether the instances get listed if the selected metrics are not available for that and the recommendations are still being listed
    # No metrics available for many instances for the ET used in the test case, hence the expected recommendation must be->no recommendations
    it 'Validate whether the instances get listed if the selected metrics are not available for that and the recommendations are still being listed', monitoring: true do
      lower_range = 2
      upper_range = 20
      metrics = [
        {
          'metricName': 'avg_mem',
          'aggregation': 'AVG',
          'lowerRange': lower_range,
          'upperRange': upper_range
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET12"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'PREV_MONTH',
        "processingFlagsMap": [
          {
            'key': 'showAll',
            'value': 'true'
          }
        ],
        'returnAllInstances': true
      }
      instance_id = ''
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      response_recommendations[:edges].each do |edge|
        if (edge[:node][:options].length() == 0 && edge[:node][:currentUtilizationStatus].eql?("NOT_ENOUGH_DATA"))
          break
        elsif (edge[:node][:options].length() != 0 && edge[:node][:currentUtilizationStatus].eql?("NOT_ENOUGH_DATA"))
          fail("Recommendations are available even though metrics are not are available for some instances")
        end
      end
      #Clean up of the ET
      delete_ET(et_id)
    end

    # Test - 13 Validate the recommendations for the instances
    # We would be choosing around 6 instances as the baseline for validating the recommendations
    # ET -> Avg - System defined , Eval duration -> 7 days
    it 'Validate recommendations for 7 days eval duration with avg system defined ET for few baseline instances', monitoring: true do
      expected_instance_recommendation_map = Hash.new
      expected_instance_recommendation_map.store('i-05bf5b602f27e881e', 'terminate')
      expected_instance_recommendation_map.store('i-034b8dc7ab92e3c7d', '')
      expected_instance_recommendation_map.store('i-03ccd6ca8d53acaae', '')
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        'evaluationDuration': 'LAST_7_DAYS',
        "processingFlagsMap": [
          {
            'key': 'showAll',
            'value': 'true'
          }
        ],
        'returnAllInstances': true
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      api_response = Hash.new
      map_1 = get_map_from_responses(response_recommendations_call_1) # key - instance_id, value - edge of an instance
      while(response_recommendations_call_1[:pageInfo][:hasNextPage]) do
        args_get_recommendations = nil
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations_2 = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
          'evaluationDuration': 'LAST_7_DAYS',
          "processingFlagsMap": [
            {
              'key': 'showAll',
              'value': 'true'
            }
          ],
          'returnAllInstances': true,
          'after': after_string_for_cursor
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations_2)
        map_2 = get_map_from_responses(response_recommendations_call_1)
        map_1 = map_1.merge!(map_2)
      end
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
        if(termination_recommendation.eql?(true) && options.size.eql?(0))
          api_response.store(instance_id, 'terminate')
        elsif(termination_recommendation.eql?(true) && !options.size.eql?(0))
          fail("Getting recommendation even with termination")
        elsif(termination_recommendation.eql?(false) && options.size.eql?(0))
          api_response.store(instance_id, '')
        elsif(termination_recommendation.eql?(false) && !options.size.eql?(0))
          first_best_rec = get_first_best_recommendation(instance_id, nil, edge)
          api_response.store(instance_id, first_best_rec)
        end
      end
      is_recommendations_matching = expected_instance_recommendation_map == api_response
      fail("Data not matching with baseline data for recommendations, expected is : #{expected_instance_recommendation_map} and actual is : #{api_response}") unless is_recommendations_matching
    end

    # Test - 14 Validate the recommendations for the instances
    # We would be choosing around 6 instances as the baseline for validating the recommendations
    # ET -> Max - System defined , Eval duration -> 30 days
    it 'Validate recommendations for 30 days eval duration with max system defined ET for few baseline instances' do
      expected_instance_recommendation_map = Hash.new
      expected_instance_recommendation_map.store('i-05bf5b602f27e881e', '')
      expected_instance_recommendation_map.store('i-034b8dc7ab92e3c7d', '')
      expected_instance_recommendation_map.store('i-03ccd6ca8d53acaae', '')

      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_max',
        'evaluationDuration': 'LAST_30_DAYS',
        "processingFlagsMap": [
          {
            'key': 'showAll',
            'value': 'true'
          },
          {
             'key': 'recommendCrossArch',
              'value': 'false'
          }
        ],
        'returnAllInstances': true
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      api_response = Hash.new
      map_1 = get_map_from_responses(response_recommendations_call_1) # key - instance_id, value - edge of an instance
      while(response_recommendations_call_1[:pageInfo][:hasNextPage]) do
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations = nil
        args_get_recommendations = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_max',
          'evaluationDuration': 'LAST_30_DAYS',
          "processingFlagsMap": [
            {
              'key': 'showAll',
              'value': 'true'
            },
            {
               'key': 'recommendCrossArch',
                'value': 'false'
             }
          ],
          'returnAllInstances': true,
          'after': after_string_for_cursor
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        map_2 = get_map_from_responses(response_recommendations_call_1)
        map_1 = map_1.merge!(map_2)
      end
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
        if(termination_recommendation.eql?(true) && options.size.eql?(0))
          api_response.store(instance_id, 'terminate')
        elsif(termination_recommendation.eql?(true) && !options.size.eql?(0))
          fail("Getting recommendation even with termination")
        elsif(termination_recommendation.eql?(false) && options.size.eql?(0))
          api_response.store(instance_id, '')
        elsif(termination_recommendation.eql?(false) && !options.size.eql?(0))
          first_best_rec = get_first_best_recommendation(instance_id, nil, edge)
          api_response.store(instance_id, first_best_rec)
        end
      end
      fail("Data not matching with baseline data for recommendations, expected is : #{expected_instance_recommendation_map} and actual is : #{api_response}") unless expected_instance_recommendation_map == api_response
    end

    # Test - 15 Validate the recommendations for the instances
    # We would be choosing around 6 instances as the baseline for validating the recommendations
    # ET -> cpu(Avg)- 2-20%, Eval duration -> 60 days
    xit 'Validate recommendations for 60 days eval duration with custom ET for few baseline instances' do
      expected_instance_recommendation_map = Hash.new
#     expected_instance_recommendation_map.store('i-05bf5b602f27e881e', 'c5a.xlarge')
#     expected_instance_recommendation_map.store('i-034b8dc7ab92e3c7d', 'c6g.xlarge')
#     expected_instance_recommendation_map.store('i-03ccd6ca8d53acaae', 'c5ad.large')
      expected_instance_recommendation_map.store('i-05bf5b602f27e881e', 't3a.nano')
      expected_instance_recommendation_map.store('i-034b8dc7ab92e3c7d', 't4g.nano')
      expected_instance_recommendation_map.store('i-03ccd6ca8d53acaae', 'c6id.large')
      metrics = [
        {
          'metricName': 'avg_cpu',
          'aggregation': 'AVG',
          'lowerRange': 2,
          'upperRange': 20
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET13"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_60_DAYS',
        "processingFlagsMap": [
          {
            'key': 'showAll',
            'value': 'true'
          }
        ],
        'returnAllInstances': true
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      map_1 = get_map_from_responses(response_recommendations_call_1) # key - instance_id, value - edge of an instance
      while(response_recommendations_call_1[:pageInfo][:hasNextPage])
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': et_id,
          'evaluationDuration': 'LAST_60_DAYS',
          "processingFlagsMap": [
            {
              'key': 'showAll',
              'value': 'true'
            }
          ],
          'returnAllInstances': true,
          'after': after_string_for_cursor
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        map_2 = get_map_from_responses(response_recommendations_call_1)
        map_1 = map_1.merge!(map_2)
      end
      api_response = Hash.new
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
        if(termination_recommendation.eql?(true) && options.size.eql?(0))
          api_response.store(instance_id, 'terminate')
        elsif(termination_recommendation.eql?(true) && !options.size.eql?(0))
          fail("Getting recommendation even with termination")
        elsif(termination_recommendation.eql?(false) && options.size.eql?(0))
          api_response.store(instance_id, '')
        elsif(termination_recommendation.eql?(false) && !options.size.eql?(0))
          first_best_rec = get_first_best_recommendation(instance_id, nil, edge)
          api_response.store(instance_id, first_best_rec)
        end
      end
      fail("Data not matching with baseline data for recommendations, expected is : #{expected_instance_recommendation_map} and actual is : #{api_response}") unless expected_instance_recommendation_map == api_response
    end

    # Test - 16 Validate the recommendations for the instances
    # We would be choosing around 10 instances as the baseline for validating the recommendations
    # ET -> mem(Max)- 2-20%, Eval duration -> 60 days
    it 'Validate recommendations for 60 days eval duration with max mem ET for few baseline instances', monitoring: true do
      expected_instance_recommendation_map = Hash.new
      expected_instance_recommendation_map.store('i-05bf5b602f27e881e', '')
      expected_instance_recommendation_map.store('i-034b8dc7ab92e3c7d', '')
      expected_instance_recommendation_map.store('i-03ccd6ca8d53acaae', 'm5ad.large')
      # expected_instance_recommendation_map.store('i-00e6e8fdafa3dc697', '')
      metrics = [
        {
          'metricName': 'max_mem',
          'aggregation': 'MAX',
          'lowerRange': 2,
          'upperRange': 20
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET1"+rand(200000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_60_DAYS',
        "processingFlagsMap": [
          {
            'key': 'showAll',
            'value': 'true'
          }
        ],
        'returnAllInstances': true,
        'after': nil,
        'sortRules': [
          {
            "field": "projectedMonthlyPrice",
            "direction": "ASC"
          }
        ]
      }

      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      api_response = Hash.new
      map_1 = get_map_from_responses(response_recommendations_call_1) # key - instance_id, value - edge of an instance
      while(response_recommendations_call_1[:pageInfo][:hasNextPage])
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': et_id,
          'evaluationDuration': 'LAST_60_DAYS',
          "processingFlagsMap": [
            {
              'key': 'showAll',
              'value': 'true'
            }
          ],
          'returnAllInstances': true,
          'after': after_string_for_cursor
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        map_2 = get_map_from_responses(response_recommendations_call_1)
        map_1 = map_1.merge!(map_2)
      end
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
        if(termination_recommendation.eql?(true) && options.size.eql?(0))
          api_response.store(instance_id, 'terminate')
        elsif(termination_recommendation.eql?(true) && !options.size.eql?(0))
          fail("Getting recommendation even with termination")
        elsif(termination_recommendation.eql?(false) && options.size.eql?(0))
          api_response.store(instance_id, '')
        elsif(termination_recommendation.eql?(false) && !options.size.eql?(0))
          first_best_rec = get_first_best_recommendation(instance_id, nil, edge)
          api_response.store(instance_id, first_best_rec)
        end
      end
      fail("Data not matching with baseline data for recommendations, expected is : #{expected_instance_recommendation_map} and actual is : #{api_response}") unless expected_instance_recommendation_map == api_response
      delete_ET(et_id)
    end

    # Test - 17 Validate the recommendations for the instances
    # We would be choosing around 6 instances as the baseline for validating the recommendations
    # ET -> mem(Avg)- 2-20%, Eval duration -> 60 days
    it 'Validate recommendations for 60 days eval duration with avg mem ET for few baseline instances', monitoring: true do
      expected_instance_recommendation_map = Hash.new
      expected_instance_recommendation_map.store('i-05bf5b602f27e881e', 't3a.micro')
      expected_instance_recommendation_map.store('i-034b8dc7ab92e3c7d', 't4g.micro')
      expected_instance_recommendation_map.store('i-03ccd6ca8d53acaae', '')
      metrics = [
        {
          'metricName': 'avg_mem',
          'aggregation': 'AVG',
          'lowerRange': 2,
          'upperRange': 20
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET2"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_60_DAYS',
        "processingFlagsMap": [
          {
            'key': 'showAll',
            'value': 'true'
          }
        ],
        'returnAllInstances': true,
        'after': nil,
        'sortRules': [
          {
            "field": "projectedMonthlyPrice",
            "direction": "ASC"
          }
        ]
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      api_response = Hash.new
      map_1 = get_map_from_responses(response_recommendations_call_1) # key - instance_id, value - edge of an instance
      while(response_recommendations_call_1[:pageInfo][:hasNextPage])
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': et_id,
          'evaluationDuration': 'LAST_60_DAYS',
          "processingFlagsMap": [
            {
              'key': 'showAll',
              'value': 'true'
            }
          ],
          'returnAllInstances': true,
          'after': after_string_for_cursor,
          'sortRules': [
             {
              "field": "projectedMonthlyPrice",
              "direction": "ASC"
            }
          ]
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        map_2 = get_map_from_responses(response_recommendations_call_1)
        map_1 = map_1.merge!(map_2)
      end
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
        if(termination_recommendation.eql?(true) && options.size.eql?(0))
          api_response.store(instance_id, 'terminate')
        elsif(termination_recommendation.eql?(true) && !options.size.eql?(0))
          fail("Getting recommendation even with termination")
        elsif(termination_recommendation.eql?(false) && options.size.eql?(0))
          api_response.store(instance_id, '')
        elsif(termination_recommendation.eql?(false) && !options.size.eql?(0))
          first_best_rec = get_first_best_recommendation(instance_id, nil, edge)
          api_response.store(instance_id, first_best_rec)
        end
      end
      fail("Data not matching with baseline data for recommendations, expected is : #{expected_instance_recommendation_map} and actual is : #{api_response}") unless expected_instance_recommendation_map == api_response
      delete_ET(et_id)
    end

    # Test - 18 Validate the recommendations for the instances
    # We would be choosing around 10 instances as the baseline for validating the recommendations
    # ET -> disk(Max)- 2-20%, Eval duration -> Previous month
    it 'Validate recommendations for 60 days eval duration with max disk ET for few baseline instances', monitoring: true do
      expected_instance_recommendation_map = Hash.new
      expected_instance_recommendation_map.store('i-0b6a60449ecf1470f', '')
      expected_instance_recommendation_map.store('i-01629125f8c66ec1d', '')
      #       expected_instance_recommendation_map.store('i-00dfc2106bdfa79df', '')
      #       expected_instance_recommendation_map.store('i-0adf713c586acfa99', '')
      # expected_instance_recommendation_map.store('i-0138a0d913fde0ebd', '')
      metrics = [
        {
          'metricName': 'max_disk',
          'aggregation': 'MAX',
          'lowerRange': 2,
          'upperRange': 20
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET3"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_60_DAYS',
        "processingFlagsMap": [
          {
            'key': 'showAll',
            'value': 'true'
          }
        ],
        'returnAllInstances': true,
        'after': nil,
        'sortRules': [
          {
            "field": "projectedMonthlyPrice",
            "direction": "DESC"
          }
        ]
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      api_response = Hash.new
      map_1 = get_map_from_responses(response_recommendations_call_1) # key - instance_id, value - edge of an instance
      while(response_recommendations_call_1[:pageInfo][:hasNextPage])
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations = nil
        args_get_recommendations = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': et_id,
          'evaluationDuration': 'LAST_60_DAYS',
          "processingFlagsMap": [
            {
              'key': 'showAll',
              'value': 'true'
            }
          ],
          'sortRules': [
            {
              "field": "projectedMonthlyPrice",
              "direction": "DESC"
            }
          ],
          'returnAllInstances': true,
          'after': after_string_for_cursor
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        map_2 = get_map_from_responses(response_recommendations_call_1)
        map_1 = map_1.merge!(map_2)
      end
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
        if(termination_recommendation.eql?(true) && options.size.eql?(0))
          api_response.store(instance_id, 'terminate')
        elsif(termination_recommendation.eql?(true) && !options.size.eql?(0))
          fail("Getting recommendation even with termination")
        elsif(termination_recommendation.eql?(false) && options.size.eql?(0))
          api_response.store(instance_id, '')
        elsif(termination_recommendation.eql?(false) && !options.size.eql?(0))
          first_best_rec = get_first_best_recommendation(instance_id, nil, edge)
          api_response.store(instance_id, first_best_rec)
        end
      end
      fail("Data not matching with baseline data for recommendations, expected is : #{expected_instance_recommendation_map} and actual is : #{api_response}") unless expected_instance_recommendation_map == api_response
      delete_ET(et_id)
    end

    # Test - 19 Validate the projected savings is equal to the sum of the savings of all the instances
    xit 'Validate the projected savings is equal to the sum of the savings of all the instances' do
      metrics = [
        {
          'metricName': 'max_disk',
          'aggregation': 'MAX',
          'lowerRange': 2,
          'upperRange': 20
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET4"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'PREV_MONTH',
        'returnAllInstances': true,
        'sortRules': [
          {
            "field": "projectedMonthlyPrice",
            "direction": "ASC"
          }
        ]
      }
      map_1 = Hash.new
      sum = 0
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      map_1 = get_map_from_responses(response_recommendations_call_1)
      File.write(map_1, "../lib/support/yml/test_map_1.json")
      while(response_recommendations_call_1[:pageInfo][:hasNextPage])
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations = nil
        args_get_recommendations = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': et_id,
          'evaluationDuration': 'LAST_60_DAYS',
          'returnAllInstances': true,
          'after': after_string_for_cursor,
          'sortRules': [
            {
              "field": "projectedMonthlyPrice",
              "direction": "ASC"
            }
          ]
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        map_2 = get_map_from_responses(response_recommendations_call_1)
        map_1 = map_1.merge!(map_2)
      end
      map_1.each_value do |edge|
        sum = sum + edge[:node][:projectedMonthlySavings]
      end
      fail("Expected savings = #{sum.round} but savings from API response = #{response_recommendations_call_1[:summary][:projectedMonthlyDifference].round}") unless sum.round.eql?(response_recommendations_call_1[:summary][:projectedMonthlyDifference].round)
      delete_ET(et_id)
    end

    # Test - 20 Validate the current monthly price is equal to the sum of the current prices of all the instances
    # Eval duration - 30 days, Custom ET - max cpu
    xit 'Validate the current monthly price is equal to the sum of the current prices of all the instances' do
      metrics = [
        {
          'metricName': 'max_cpu',
          'aggregation': 'MAX',
          'lowerRange': 2,
          'upperRange': 20
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET14"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_30_DAYS',
        'returnAllInstances': true,
        'after': nil,
        'sortRules': [
          {
          "field": "projectedMonthlyPrice",
        "direction": "DESC"
      }
      ]
      }
      map_1 = Hash.new
      sum = 0
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      map_1 = get_map_from_responses(response_recommendations_call_1)
      while(response_recommendations_call_1[:pageInfo][:hasNextPage])
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations = nil
        args_get_recommendations = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': et_id,
          'evaluationDuration': 'LAST_60_DAYS',
          'returnAllInstances': true,
          'after': after_string_for_cursor,
          'sortRules': [
            {
              "field": "projectedMonthlyPrice",
              "direction": "ASC"
            }
          ]
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        map_2 = get_map_from_responses(response_recommendations_call_1)
        map_1 = map_1.merge(map_2)
      end
      puts "map_1 size is #{map_1.size}"
      map_1.each_value do |edge|
        sum = sum + edge[:node][:currentMonthlyCost]
      end
      fail("Expected current monthly cost = #{sum.round} but current cost from API response = #{response_recommendations_call_1[:summary][:monthlyCost].round}") unless sum.round.eql?(response_recommendations_call_1[:summary][:monthlyCost].round)
      delete_ET(et_id)
    end

    # Test - 21 Validate the utilisation summary from the API
    it 'Validate the utilisation summary from the API', monitoring: true do # Same test we are doing it on Summary API Cases
      metrics = [
        {
          'metricName': 'max_cpu',
          'aggregation': 'MAX',
          'lowerRange': 2,
          'upperRange': 20
        },
        {
          'metricName': 'avg_mem',
          'aggregation': 'AVG',
          'lowerRange': 2,
          'upperRange': 20
        },
        {
          'metricName': 'avg_disk',
          'aggregation': 'AVG',
          'lowerRange': 2,
          'upperRange': 10
        }
      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_Delete_ET5"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_60_DAYS',
        "processingFlagsMap": [
          {
            'key': 'showAll',
            'value': 'true'
          }
        ],
        'returnAllInstances': true,
        'after': nil,
        'sortRules': [
          {
            "field": "projectedMonthlyPrice",
            "direction": "DESC"
          }
        ]
      }
      result_map = Hash.new
      current_price_calculated = 0
      projected_price_calculated = 0
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      result_map = get_map_from_responses(response_recommendations_call_1)
      while(response_recommendations_call_1[:pageInfo][:hasNextPage].eql?(true))
        args_get_recommendations = nil
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': et_id,
          'evaluationDuration': 'LAST_60_DAYS',
          "processingFlagsMap": [
            {
              'key': 'showAll',
              'value': 'true'
            }
          ],
          'returnAllInstances': true,
          'after': after_string_for_cursor,
          'sortRules': [
            {
              "field": "projectedMonthlyPrice",
              "direction": "DESC"
            }
          ]
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        result_map = result_map.merge!(get_map_from_responses(response_recommendations_call_1))
      end
      args_get_summary = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_60_DAYS',
        "processingFlagsMap": [
          {
            'key': 'showAll',
            'value': 'true'
          }
        ],
      }
      response_summary = rightsizing_recommendations_summary(args_get_summary)
      result_map.each_value do |edge|
        current_price_calculated = current_price_calculated + edge[:node][:currentMonthlyPrice]
        projected_price_calculated = projected_price_calculated + edge[:node][:projectedMonthlyPrice]
      end
      current_savings_calculated =  current_price_calculated - projected_price_calculated
      saving_value_from_api = response_summary[:projectedMonthlyDifference]
      is_savings_matching = current_savings_calculated.round.eql?(saving_value_from_api.round)
      is_projected_price_matching = projected_price_calculated.round.eql?(response_summary[:projectedMonthlyPrice].round)
      is_current_price_matching = current_price_calculated.round.eql?(response_summary[:monthlyCost].round)
      #efficiency_score_from_api = response_summary[:efficiencyScore]
      #current_efficiency_score_calculated = response_recommendations_call_1[:utilizationStatusSummary][:goodFit].to_f / response_summary[:totalTargetResources].to_f * 100
      #is_efficiency_score_matching = efficiency_score_from_api.round.eql?(current_efficiency_score_calculated.round)
      fail("Summary is not matching") unless is_savings_matching && is_projected_price_matching && is_current_price_matching
      #&& is_efficiency_score_matching
      delete_ET(et_id)
    end

    # Test - 22 Validate the updated recommendation with new efficiency target and evaluation duration
    it 'Validate recommendations get updated with new efficiency target and evaluation duration', monitoring: true do
      expected_instance_recommendation_map = Hash.new
      expected_instance_recommendation_map.store('i-05bf5b602f27e881e', '')
      expected_instance_recommendation_map.store('i-034b8dc7ab92e3c7d', 'c6g.xlarge')
      expected_instance_recommendation_map.store('i-03ccd6ca8d53acaae', 'm5ad.large')

      metrics = [
        {
          'metricName': 'max_mem',
          'aggregation': 'MAX',
          'lowerRange': 2,
          'upperRange': 20
        },
        {
          'metricName': 'avg_cpu',
          'aggregation': 'AVG',
          'lowerRange': 2,
          'upperRange': 20
        }
      ]
      name = "Test_Delete_ET6"+rand(20000).to_s
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': name
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'PREV_MONTH',
        'returnAllInstances': true,
        'sortRules': [
          {
            "field": "projectedMonthlyPrice",
            "direction": "DESC"
          }
        ]
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      api_response = Hash.new
      map_1 = get_map_from_responses(response_recommendations_call_1) # key - instance_id, value - edge of an instance
      while(response_recommendations_call_1[:pageInfo][:hasNextPage].eql?(true))
        args_get_recommendations = nil
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': et_id,
          'evaluationDuration': 'PREV_MONTH',
          'returnAllInstances': true,
          'after': after_string_for_cursor,
          'sortRules': [
            {
              "field": "projectedMonthlyPrice",
              "direction": "ASC"
            }
          ]
        }
        response_recommendations_call_1 = nil
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        map_2 = get_map_from_responses(response_recommendations_call_1)
        map_1 = map_1.merge!(map_2)
      end
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
        if(termination_recommendation.eql?(true) && options.size.eql?(0))
          api_response.store(instance_id, 'terminate')
        elsif(termination_recommendation.eql?(true) && !options.size.eql?(0))
          fail("Getting recommendation even with termination")
        elsif(termination_recommendation.eql?(false) && options.size.eql?(0))
          api_response.store(instance_id, '')
        elsif(termination_recommendation.eql?(false) && !options.size.eql?(0))
          first_best_rec = get_first_best_recommendation(instance_id, nil, edge)
          api_response.store(instance_id, first_best_rec)
        end
      end
      fail("Data not matching with baseline data for recommendations, expected is : #{expected_instance_recommendation_map} and actual is : #{api_response}") unless expected_instance_recommendation_map == api_response

      # Updating the ET with new data
      updated_metrics = [
        {
          'metricName': 'max_cpu',
          'aggregation': 'MAX',
          'lowerRange': 2,
          'upperRange': 20
        }
      ]
      wrap_request do
        variables = {
          'input': {
            'id': et_id,
            'metrics': updated_metrics,
            'name': "updatedName#{name}"
          }
        }
        graphql url(service: service),
                format_request(
                  :updateRightsizingEfficiencyTarget,
                  service: service
                ),
                variables, headers
      end
      api_response_2 = Hash.new
      args_get_recommendations = nil
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_60_DAYS',
        'returnAllInstances': true,
        'sortRules': [
          {
            "field": "projectedMonthlyPrice",
            "direction": "DESC"
          }
        ]
      }
      response_recommendations_call_2 = rightsizing_recommendations(args_get_recommendations)
      result_map = get_map_from_responses(response_recommendations_call_2)
      after_string_for_cursor = response_recommendations_call_2[:pageInfo][:endCursor]
      while(response_recommendations_call_2[:pageInfo][:hasNextPage].eql?(true))
        args_get_recommendations = nil
        args_get_recommendations = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': et_id,
          'evaluationDuration': 'LAST_60_DAYS',
          'sortRules': [
            {
              "field": "projectedMonthlyPrice",
              "direction": "DESC"
            }
          ],
          'returnAllInstances': true,
          'after': after_string_for_cursor
        }
        response_recommendations_call_2 = nil
        response_recommendations_call_2 = rightsizing_recommendations(args_get_recommendations)
        result_map = result_map.merge!(get_map_from_responses(response_recommendations_call_2))
      end
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = result_map[instance_id]
        options = get_recommendation_for_anInstance(edge)
        termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
        if(termination_recommendation.eql?(true) && options.size.eql?(0))
          api_response_2.store(instance_id, 'terminate')
        elsif(termination_recommendation.eql?(true) && !options.size.eql?(0))
          fail("Getting recommendation even with termination")
        elsif(termination_recommendation.eql?(false) && options.size.eql?(0))
          api_response_2.store(instance_id, '')
        elsif(termination_recommendation.eql?(false) && !options.size.eql?(0))
          first_best_rec = get_first_best_recommendation(instance_id, nil, edge)
          api_response_2.store(instance_id, first_best_rec)
        end
      end
      fail("Getting same recommendation even with updated efficiency target") if api_response.eql?(api_response_2)
      delete_ET(et_id)
    end

    # Test - 23 Validate the recommendations for the recommend burstables
    # We would be choosing only instance as the baseline for validating the recommendations
    # ET -> cpu avg- 2-20%, Eval duration -> Last 60 days
    # Expected 1st best recommendation = t4g.micro
    xit 'Validate recommendations for an instance with recommend burstables flag as true' do
      expected_instance_recommendation_map = Hash.new
      #expected_instance_recommendation_map.store('i-034b8dc7ab92e3c7d', 't4g.micro')
      expected_instance_recommendation_map.store('i-034b8dc7ab92e3c7d', 't4g.nano')
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        'evaluationDuration': 'LAST_30_DAYS',
        "processingFlagsMap": [
          {
            'key': 'recommendBurstable',
            'value': 'true'
          }
        ],
        'returnAllInstances': true
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      api_response = Hash.new
      map_1 = get_map_from_responses(response_recommendations_call_1) # key - instance_id, value - edge of an instance
      while(response_recommendations_call_1[:pageInfo][:hasNextPage])
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations = nil
        args_get_recommendations = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
          'evaluationDuration': 'LAST_30_DAYS',
          'returnAllInstances': true,
          'after': after_string_for_cursor,
          "processingFlagsMap": [
            {
              'key': 'recommendBurstable',
              'value': 'true'
            }
          ]
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        map_2 = get_map_from_responses(response_recommendations_call_1)
        map_1 = map_1.merge!(map_2)
      end
      expected_instance_recommendation_map.each_key do |instance_id|
        edge = map_1[instance_id]
        options = get_recommendation_for_anInstance(edge)
        best_recommendation_1 = get_first_best_recommendation(instance_id, nil, edge)
        fail("Not getting burstable recommendation") unless best_recommendation_1.eql?(expected_instance_recommendation_map[instance_id])
      end
    end

    # Test - 24 Validate the recommendations for the recommend burstables
    # We would be choosing only instance as the baseline for validating the recommendations
    # ET -> disk(Max)- 2-20%, Eval duration -> Last 60 days
    # Expected 1st best recommendation =
    it 'Validate recommendations for an instance with recommend burstables flag as false', monitoring: true do
      expected_instance_recommendation_map = Hash.new
      instance_id="i-034b8dc7ab92e3c7d"
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        'evaluationDuration': 'LAST_30_DAYS',
        'processingFlagsMap': [
          {
            'key': 'recommendBurstable',
            'value': 'false'
          }
        ],
        'returnAllInstances': true
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      api_response = Hash.new
      map_1 = get_map_from_responses(response_recommendations_call_1) # key - instance_id, value - edge of an instance
      while(response_recommendations_call_1[:pageInfo][:hasNextPage])
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations = nil
        args_get_recommendations = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
          'evaluationDuration': 'LAST_30_DAYS',
          'returnAllInstances': true,
          'after': after_string_for_cursor,
          'processingFlagsMap': [
            {
              'key': 'recommendBurstable',
              'value': 'false'
            }
          ]
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
        map_2 = get_map_from_responses(response_recommendations_call_1)
        map_1 = map_1.merge!(map_2)
      end
        edge = map_1[instance_id]
        best_recommendation_1 = get_first_best_recommendation(instance_id, nil, edge)
        getting_burstables = best_recommendation_1.start_with?('t')
        fail("Still getting burstables recommendation ") if getting_burstables
      end



    # Test - 25 Validate the recommendations for the instances
    # We would be verifying the terminate recommendations, and no recommendation and recommendation instance
    # ET -> Avg - System defined , Eval duration -> 7 days showALL -false
    it 'Validate recommendations for 7 days eval duration with avg system defined ET with showAll:false terminate recommendation', monitoring: true do
      instance_id='i-0b6a60449ecf1470'
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
        'evaluationDuration': 'LAST_7_DAYS',
        "processingFlagsMap": [
          {
            'key': 'showAll',
            'value': 'false'
          }
        ],
        'returnAllInstances': false
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      map_1 = get_map_from_responses(response_recommendations_call_1) # key - instance_id, value - edge of an instance
      while(response_recommendations_call_1[:pageInfo][:hasNextPage]) do
        after_string_for_cursor = response_recommendations_call_1[:pageInfo][:endCursor]
        args_get_recommendations_2 = {
          'assetType': 'AWS_EC2',
          'efficiencyTarget': 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg',
          'evaluationDuration': 'LAST_7_DAYS',
          "processingFlagsMap": [
            {
              'key': 'showAll',
              'value': 'false'
            }
          ],
          'returnAllInstances': false,
          'after': after_string_for_cursor
        }
        response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations_2)
        map_2 = get_map_from_responses(response_recommendations_call_1)
        map_1 = map_1.merge!(map_2)
      end
      edge = map_1[instance_id]
      termination_recommendation = get_current_termination_recommendation_of_instance(instance_id, nil, edge)
      if(termination_recommendation.eql?(false))
        break
        fail("terminate recommendation are not visible")
      end
      map_1.each_value do |edge|
        if (edge[:node][:options].length() == 0 && edge[:node][:currentUtilizationStatus].eql?("NOT_ENOUGH_DATA"))
          break
          fail("Instances with no recommendations are visible")
        end
      end
    end

    # Test - 26 : Validate if a user is able to get the recommendation for network out metric
    # ET - netoworkout 2-80 , Duration - Last 60 days
    xit 'Validate the rightsizing recommendation for network out metrics' do
      metrics = [
        {
          'metricName': 'avg_network_out',
          'aggregation': 'AVG',
          'lowerRange': 2,
          'upperRange': 80
        }

      ]
      args = {
        'assetType': 'AWS_EC2',
        'metrics': metrics,
        'name': "Test_networkout"+rand(20000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      args_get_recommendations = {
        'assetType': 'AWS_EC2',
        'efficiencyTarget': et_id,
        'evaluationDuration': 'LAST_60_DAYS',
        "processingFlagsMap": [
          {
            'key': 'showAll',
            'value': 'true'
          }
        ],
        'returnAllInstances': true
      }
      response_recommendations = rightsizing_recommendations(args_get_recommendations)
      instance_id = "c6id.large"
      best_recommendation = get_first_best_recommendation(instance_id, response_recommendations, nil )
      fail ("Recommendations are not matching the expectation") unless best_recommendation.eql?("m5dn.large")
      #Clean up of the ET
      delete_ET(et_id)
    end


    #Method to get the days in a month
    COMMON_YEAR_DAYS_IN_MONTH = [nil, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]

    def days_in_month(month, year = Time.now.year)
      return 29 if month == 2 && Date.gregorian_leap?(year)
      COMMON_YEAR_DAYS_IN_MONTH[month]
    end

    # Method to get the expected utilization status
    def get_expected_utilization_status(current_value, lower_range, upper_range)
      status = ''
      if current_value.eql?(nil)
        status = 'NOT_ENOUGH_DATA'
      elsif current_value < lower_range
        status = 'UNDER_TARGET'
      elsif current_value> upper_range
        status = 'OVER_TARGET'
      elsif current_value >= lower_range && current_value <= upper_range
        status = 'GOOD_FIT'
      end
      status
    end

    # Method to get the type of an instance
    def get_instance_type(edge)
      type = edge[:node][:targetAsset][:instanceType][:name]
      return type
    end

    # Method to get the name of an instance
    def get_instance_name(edge)
      name = edge[:node][:targetAsset][:name]
      return name
    end

    # Map contains key as instance id and value as the data of each node
    def get_map_from_responses(recommendations_response)
      responses_map = Hash.new
      recommendations_response[:edges].each do |edge|
        responses_map.store(get_instance_id(edge), edge)
      end
      responses_map
    end

    # Upload the mock file to S3
    xit 'Validate uploading the mock file' do
      file_name = "aws_instance_performance_#{Date.today.prev_day}"
      File.rename("lib/support/yml/efficiency_targets_copy.yml", "lib/support/yml/#{file_name}")
      puts file_name
      FILE = File.expand_path("../lib/support/yml/#{file_name}", __dir__)
      wrapper = Aws::S3::Object.new('cht-svc-rightsizing-framework-staging-virginia-1', FILE)
      return unless wrapper.upload_file(FILE)
    end

    # Returns the current metrics value of the instance passed and the metric id
    def get_metrics_value_based_on_userInput(instance_id, recommendation_response, metricId)
      current_metrics = get_current_metrics(instance_id, recommendation_response)
      val = 0
      current_metrics.each do |metric|
        if metric[:metricName].eql?(metricId)
          val = metric[:value]
        end
      end
      val
    end

    # Returns the current metrics status of the instance passed
    def get_metrics_status_based_on_userInput(instance_id, recommendation_response, metricId)
      current_metrics = get_current_metrics(instance_id, recommendation_response)
      status = ""
      current_metrics.each do |metric|
        if metric[:metricName].eql?(metricId)
          status = metric[:status]
        end
      end
      status
    end

    # Returns the current metrics of the instance passed
    def get_current_metrics(instance_id, recommendation_response)
      map = Hash.new
      recommendation_response[:edges].each do |edge|
        map.store(get_instance_id(edge), edge[:node][:currentMetrics])
      end
      return map[instance_id]
    end

    # Returns the current utilization status of the instance
    def get_current_utilization_status_of_instance(instance_id, recommendation_response)
      instance_values_fromMap = get_details(instance_id, recommendation_response)
      instance_values_fromMap[:currentUtilizationStatus]
    end

    # Returns the termination recommendation of the instance
    def get_current_termination_recommendation_of_instance(instance_id, recommendation_response, edge)
      if(!recommendation_response.eql?(nil))
        instance_values_fromMap = get_details(instance_id, recommendation_response)
        return instance_values_fromMap["terminateRecommendation"]
      elsif(recommendation_response.eql?(nil) && !edge.eql?(nil))
        return edge[:node][:terminateRecommendation]
      end
    end

    # Returns the first best recommendation of the instance if there is any recommendation
    def get_first_best_recommendation(instance_id, recommendation_response, edge)
      if(!recommendation_response.eql?(nil))
        instance_values_fromMap = get_details(instance_id, recommendation_response)
        recommendation_options = instance_values_fromMap["options"]
        first_best_recommendation = ''
        recommendation_options.each do |option|
          if option[:bestFit].eql?(true)
            first_best_recommendation = option[:instanceType][:name]
          end
        end
        return first_best_recommendation
      elsif(recommendation_response.eql?(nil) && !edge.eql?(nil))
        if(edge[:node][:options].size.eql?(0))
          return "No recommendations"
        else
          edge[:node][:options].each do |option|
            if(option[:bestFit].eql?(true))
              return option[:instanceType][:name]
            end
          end
        end
      end
    end

    # Method to get the recommendations for an instance
    def get_recommendation_for_anInstance(edge)
      return edge[:node][:options]
    end

    # Returns the summary of the instance passed
    def get_details(instance_id, response)
      map_data = get_data_from_response(response)
      return map_data[instance_id]
    end

    # Returns map of key value pair where key is the instance id and value is the another hashmap of utilization status, termination recommendation and recommendation options.
    def get_data_from_response(recommendations_response)
      map = Hash.new
      recommendations_response[:edges].each do |edge|
        data_list = Hash.new
        data_list.store("currentUtilizationStatus", edge[:node][:currentUtilizationStatus])
        data_list.store("terminateRecommendation", edge[:node][:terminateRecommendation])
        data_list.store("options", edge[:node][:options])
        map.store(get_instance_id(edge), data_list)
      end
      map
    end

    # Returns the instance id of the node passed
    def get_instance_id(edge)
      instance_id = edge[:node][:targetAsset][:awsInstanceId]
      return instance_id
    end
    end
  end

