# Test cases related to getting the recommendations with filters

require 'spec_helper'
require 'date'

module RightsizingFrameworkTest
  describe 'RightsizingFramework GetRecommendations with filters' do
    include_context 'request metadata'

    # Test - 1 : Validate the filter utilization status
    it 'Validate filter for utilization summary', monitoring: true do # Duplicate from recommendation test cases
      lower_range = 2
      upper_range = 5
      metrics = [
        {
          metricName: 'avg_cpu',
          aggregation: 'AVG',
          lowerRange: lower_range,
          upperRange: upper_range
        }
      ]
      args = {
        assetType: 'AWS_EC2',
        metrics: metrics,
        name: 'Test_Delete_ET' + rand(20_000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      other_criteria = [
        {
          name: 'currentUtilizationStatus',
          values: %w[GOOD_FIT]
        }
      ]
      args_get_recommendations = {
        assetType: 'AWS_EC2',
        efficiencyTarget: et_id,
        evaluationDuration: 'LAST_60_DAYS',
        filterOptions: {
          otherCriteria: other_criteria
        },
        returnAllInstances: true
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      raise('utilization filters are not applied') unless response_recommendations_call_1[:edges].size < 100

      # Clean up of the ET
      delete_ET(et_id)
    end

    # Test - 2 : Validate the filter with multiple utilization status
    it 'Validate the filter with multiple utilization status', monitoring: true do #Duplicate from recommendation test cases
      lower_range = 2
      upper_range = 5
      metrics = [
        {
          metricName: 'avg_cpu',
          aggregation: 'AVG',
          lowerRange: lower_range,
          upperRange: upper_range
        }
      ]
      args = {
        assetType: 'AWS_EC2',
        metrics: metrics,
        name: 'Test_Delete_ET' + rand(20_000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      other_criteria = [
        {
          name: 'currentUtilizationStatus',
          values: %w[GOOD_FIT UNDER_TARGET]
        }
      ]
      args_get_recommendations = {
        assetType: 'AWS_EC2',
        efficiencyTarget: et_id,
        evaluationDuration: 'LAST_60_DAYS',
        filterOptions: {
          otherCriteria: other_criteria
        },
        returnAllInstances: true
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      expect(response_recommendations_call_1[:edges].size <= 15)

      # Clean up of the ET
      delete_ET(et_id)
    end

    # Test - 3 : Validate the filter with multiple utilization status and account IDs
    it 'Validate the filter with multiple utilization status and account IDs' do
      lower_range = 2
      upper_range = 5
      metrics = [
        {
          metricName: 'avg_cpu',
          aggregation: 'AVG',
          lowerRange: lower_range,
          upperRange: upper_range
        }
      ]
      args = {
        assetType: 'AWS_EC2',
        metrics: metrics,
        name: 'Test_Delete_ET' + rand(20_000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      other_criteria = [
        {
          name: 'currentUtilizationStatus',
          values: %w[GOOD_FIT UNDER_TARGET]
        },
        {
          name: 'aws_account_id',
          values: %w[081309885503 781971168076 844584796563]
        }
      ]
      args_get_recommendations = {
        assetType: 'AWS_EC2',
        efficiencyTarget: et_id,
        evaluationDuration: 'LAST_60_DAYS',
        filterOptions: {
          otherCriteria: other_criteria
        },
        returnAllInstances: true
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      raise('utilization filters are not applied') if response_recommendations_call_1[:edges].size <= 15

      # Clean up of the ET
      delete_ET(et_id)
    end

    # Test - 4 : Validate the filter with multiple utilization status and account IDs which does not have instances in it
    it 'Validate the filter with multiple utilization status and account IDs which does not have instances in it', monitoring: true do
      lower_range = 2
      upper_range = 5
      metrics = [
        {
          metricName: 'avg_cpu',
          aggregation: 'AVG',
          lowerRange: lower_range,
          upperRange: upper_range
        }
      ]
      args = {
        assetType: 'AWS_EC2',
        metrics: metrics,
        name: 'Test_Delete_ET' + rand(20_000).to_s
      }
      response = create_rightsizing_efficiency_target(args)
      et_id = response[:efficiencyTarget][:id]
      other_criteria = [
        {
          name: 'currentUtilizationStatus',
          values: %w[GOOD_FIT UNDER_TARGET]
        },
        {
          name: 'aws_account_id',
          values: %w[081309885503 781971168076]
        }
      ]
      args_get_recommendations = {
        assetType: 'AWS_EC2',
        efficiencyTarget: et_id,
        evaluationDuration: 'LAST_60_DAYS',
        filterOptions: {
          otherCriteria: other_criteria
        },
        returnAllInstances: true
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      raise('utilization filters are not applied') if response_recommendations_call_1[:edges].size > 15

      # Clean up of the ET
      delete_ET(et_id)
    end
    #Test 5 validate account filter by checking account in response of instances returned
    it 'validate the filter of azure subscription id value in response' do
      args_get_recommendations = {
        "assetType": "AZURE_VM",
        "efficiencyTarget": "crn:0:rightsizing/AZURE_VM_EfficiencyTarget:system_avg",
        "evaluationDuration": "LAST_7_DAYS",
        "processingFlagsMap": [
          {
            "key": "recommendBurstable",
            "value": "true"
          }
        ],
        "filterOptions": {
          "otherCriteria": [
            {
              "name": "azure_subscription_id",
              "values": [
                "c0ef2db4-fada-4539-a6de-8bedd2a0743f" #subscription-
              ]
            }
          ]
        },
        "returnAllInstances": true
      }
      response_recommendations_call_1 = rightsizing_recommendations(args_get_recommendations)
      expect(response_recommendations_call_1[:edges].all?{|x| x[:node][:targetAsset][:subscription][:name].eql?('redshirt Dev/Test') && x[:node][:targetAsset][:azureInstanceId].include?(args_get_recommendations[:filterOptions][:otherCriteria][0][:values][0]) }).to be_truthy
    end
  end
end
