#### GET ####
# Test cases related to GETTING the list of efficiency targets
require 'spec_helper'

module RightsizingFrameworkTest
  @generated_list_of_efficiency_targets = 'lib/support/recommendations/response.json'
  @test_file = 'lib/support/recommendations/test.json'

  # Test 1 - Get the list of all the rightsizing efficiency targets
  describe 'Gets the list of efficiency targets' do
    include_context 'request metadata'

    # Test 1 - Check if the API is reachable - to be added in smoke suite
    it 'Validate the Api is reachable for get list of efficiency targets', monitoring: true do
      # Create two efficiency targets first and then validate if the same has been fetched.
      assetType = Array.[]('AWS_EC2', 'AWS_RDS','AZURE_VM', 'AWS_EBS')
      assetType.each do |asset|
      variables = { assetType: assetType }
      response = rightsizing_efficiency_targets(variables)
      k = response[:edges]
      ids = []
      k.each do |item|
        ids.push(item[:node][:id])
      end
      #puts " The response from the api call is : #{k}"
      #puts "ids are : #{ids}"
      #puts "ids size is: #{ids.size}"
      raise('Not getting all the efficiency target IDs') if ids.size < 2
      end
      end

    @AfterTest
    # Test 2 - Create an efficiency target and validate if its one of the responses
    # ToDo - Add the clean up of the added efficiency targets
    it 'Validate getting the efficiency targets as response', monitoring: true do
      et_name = 'Test_get_et_' + rand(20_000).to_s
      args =  {
        assetType: 'AWS_EC2',
        metrics: [
          {
            metricName: 'avg_cpu',
            aggregation: 'AVG',
            lowerRange: 30,
            upperRange: 70
          }
        ],
        name: et_name
      }
      created_response = create_rightsizing_efficiency_target(args)
      id = created_response[:efficiencyTarget][:id]
      variables = { assetType: 'AWS_EC2' }
      json_response = rightsizing_efficiency_targets(variables)
      validate_get_list_of_efficiency_target_API(json_response, id)
    end

    @BeforeTest
    # Test 3 - Test case to validate there are only two default efficiency targets
    it 'Validate there are only two default efficiency targets', monitoring: true do
      variables = { assetType: 'AWS_EC2' }
      json_response = rightsizing_efficiency_targets(variables)

      edges = json_response[:edges]
      ids = []
      names =[]
      edges.each do |edge|
        ids.push(edge[:node][:id])
        names.push(edge[:node][:name])
      end
      name1 = 'Average Metrics'
      name2 = 'Maximum Metrics'
      system_definedET1 = 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_max'
      system_definedET2 = 'crn:0:rightsizing/AWS_EC2_EfficiencyTarget:system_avg'
      count = 0
      ids.each do |id|
        count += 1 if id.eql?(system_definedET1) || id.eql?(system_definedET2)
      end
      countname=0
      names.each do |name|
        countname +=1 if name.eql?(name1) || name.eql?(name2)
      end
      raise('System defined efficiency targets not present as per the match') unless count.eql?(2)
      raise('System defined efficiency target names are not two') unless countname.eql?(2)

    end
  end
end
